import * as tslib_1 from "tslib";
import { ContentTypes, FetchHookTypes } from '../types';
import { ModulePrefix } from '../index';
import { ContentTypeToString } from '../util/Filters';
import Meta from '../plugin/inheritable/Meta';
var LoaderRequestToFetches = function (slug, store) {
    var requests = [];
    for (var _i = 0, slug_1 = slug; _i < slug_1.length; _i++) {
        var requestElement = slug_1[_i];
        if (typeof requestElement === 'string') {
            requests.push(store.dispatch(ModulePrefix + "_page/load", {
                slug: requestElement,
                type: ContentTypes.Page
            }));
        }
        else {
            var slug_2 = requestElement.slug;
            var dataName = slug_2;
            if ('dataName' in requestElement) {
                dataName = requestElement.dataName;
            }
            var type = 'post' in requestElement && requestElement.post === true
                ? 'post'
                : 'page';
            requests.push(store.dispatch(ModulePrefix + "_" + type + "/load", {
                slug: slug_2,
                type: type === 'post'
                    ? ContentTypes.Post
                    : ContentTypes.Page
            }));
        }
    }
    return requests;
};
export default (function (slug, createdOrAsync) {
    var _a, _b;
    if (createdOrAsync === void 0) { createdOrAsync = FetchHookTypes.Created; }
    var metaSource;
    if (!slug || slug.length < 1) {
        throw new Error('Slug no provided');
    }
    var mixin = {};
    var computed = {};
    if (createdOrAsync === FetchHookTypes.Created) {
        // Appears in Vue version
        if (typeof slug === 'string') {
            metaSource = slug;
            computed = tslib_1.__assign({}, computed, (_a = {}, _a[slug] = function () {
                var type = ContentTypeToString(ContentTypes.Page);
                return this.$store.state[ModulePrefix + "_" + type][type][slug]
                    ? this.$store.state[ModulePrefix + "_" + type][type][slug]
                    : null;
            }, _a));
        }
        else if (Array.isArray(slug)) {
            var _loop_1 = function (requestElement) {
                var _a, _b;
                if (typeof requestElement === 'string') {
                    if (!metaSource) {
                        metaSource = requestElement;
                    }
                    computed = tslib_1.__assign({}, computed, (_a = {}, _a[requestElement] = function () {
                        var type = ContentTypeToString(ContentTypes.Page);
                        return this.$store.state[ModulePrefix + "_" + type][type][requestElement]
                            ? this.$store.state[ModulePrefix + "_" + type][type][requestElement]
                            : null;
                    }, _a));
                }
                else {
                    var slug_4 = requestElement.slug;
                    var dataName_1 = slug_4;
                    if ('dataName' in requestElement) {
                        dataName_1 = requestElement.dataName;
                    }
                    if (('meta' in requestElement && requestElement.meta === true) || !metaSource) {
                        metaSource = dataName_1;
                    }
                    var contentType_1 = 'post' in requestElement && requestElement.post === true
                        ? ContentTypes.Post
                        : ContentTypes.Page;
                    computed = tslib_1.__assign({}, computed, (_b = {}, _b[dataName_1] = function () {
                        var type = ContentTypeToString(contentType_1);
                        return this.$store.state[ModulePrefix + "_" + type][type][slug_4]
                            ? this.$store.state[ModulePrefix + "_" + type][type][slug_4]
                            : null;
                    }, _b));
                }
            };
            // slug - array of request elements in this example
            for (var _i = 0, slug_3 = slug; _i < slug_3.length; _i++) {
                var requestElement = slug_3[_i];
                _loop_1(requestElement);
            }
        }
        else {
            var requestElement = slug;
            var slugName_1 = requestElement.slug;
            var dataName_2 = slugName_1;
            if ('dataName' in requestElement) {
                dataName_2 = requestElement.dataName;
            }
            if (('meta' in requestElement && requestElement.meta === true) || !metaSource) {
                metaSource = dataName_2;
            }
            var contentType_2 = 'post' in requestElement && requestElement.post === true
                ? ContentTypes.Post
                : ContentTypes.Page;
            computed = tslib_1.__assign({}, computed, (_b = {}, _b[dataName_2] = function () {
                var type = ContentTypeToString(contentType_2);
                return this.$store.state[ModulePrefix + "_" + type][type][slugName_1]
                    ? this.$store.state[ModulePrefix + "_" + type][type][slugName_1]
                    : null;
            }, _b));
        }
        // mixin = { computed }
        mixin = {
            computed: computed,
            created: function () {
                return tslib_1.__awaiter(this, void 0, void 0, function () {
                    var requests, requestElement, slugName, type;
                    return tslib_1.__generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                requests = [];
                                if (typeof slug === 'string') {
                                    requests.push(this.$store.dispatch(ModulePrefix + "_page/load", {
                                        slug: slug,
                                        type: ContentTypes.Page
                                    }));
                                }
                                else if (Array.isArray(slug)) {
                                    // slug - array of request elements in this example
                                    requests.push.apply(requests, LoaderRequestToFetches(slug, this.$store));
                                }
                                else {
                                    requestElement = slug;
                                    slugName = requestElement.slug;
                                    type = 'post' in requestElement && requestElement.post === true
                                        ? 'post'
                                        : 'page';
                                    requests.push(this.$store.dispatch(ModulePrefix + "_" + type + "/load", {
                                        slug: slugName,
                                        type: type === 'post'
                                            ? ContentTypes.Post
                                            : ContentTypes.Page
                                    }));
                                }
                                return [4 /*yield*/, Promise.all(requests)];
                            case 1:
                                _a.sent();
                                return [2 /*return*/];
                        }
                    });
                });
            }
        };
    }
    else if (createdOrAsync === FetchHookTypes.AsyncData) {
        // Appears in Nuxt version
        mixin.asyncData = function (_a) {
            var store = _a.store;
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var requests, requestElement, slugName, type, asyncData, _i, slug_5, requestElement, slug_6, dataName, type, requestElement, slugName, dataName, type;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            requests = [];
                            if (typeof slug === 'string') {
                                requests.push(store.dispatch(ModulePrefix + "_page/load", {
                                    slug: slug,
                                    type: ContentTypes.Page
                                }));
                            }
                            else if (Array.isArray(slug)) {
                                // slug - array of request elements in this example
                                requests.push.apply(requests, LoaderRequestToFetches(slug, store));
                            }
                            else {
                                requestElement = slug;
                                slugName = requestElement.slug;
                                type = 'post' in requestElement && requestElement.post === true
                                    ? 'post'
                                    : 'page';
                                requests.push(store.dispatch(ModulePrefix + "_" + type + "/load", {
                                    slug: slugName,
                                    type: type === 'post'
                                        ? ContentTypes.Post
                                        : ContentTypes.Page
                                }));
                            }
                            return [4 /*yield*/, Promise.all(requests)];
                        case 1:
                            _b.sent();
                            asyncData = {};
                            if (typeof slug === 'string') {
                                metaSource = slug;
                                asyncData[slug] = store.state[ModulePrefix + "_page"].page[slug]
                                    ? store.state[ModulePrefix + "_page"].page[slug]
                                    : null;
                            }
                            else if (Array.isArray(slug)) {
                                for (_i = 0, slug_5 = slug; _i < slug_5.length; _i++) {
                                    requestElement = slug_5[_i];
                                    if (typeof requestElement === 'string') {
                                        asyncData[requestElement] = store.state[ModulePrefix + "_page"].page[requestElement]
                                            ? store.state[ModulePrefix + "_page"].page[requestElement]
                                            : null;
                                        if (!metaSource) {
                                            metaSource = requestElement;
                                        }
                                    }
                                    else {
                                        slug_6 = requestElement.slug;
                                        dataName = slug_6;
                                        if ('dataName' in requestElement) {
                                            dataName = requestElement.dataName;
                                        }
                                        type = 'post' in requestElement && requestElement.post === true
                                            ? 'post'
                                            : 'page';
                                        if (('meta' in requestElement && requestElement.meta === true) || !metaSource) {
                                            metaSource = dataName;
                                        }
                                        asyncData[dataName] = store.state[ModulePrefix + "_" + type][type][slug_6]
                                            ? store.state[ModulePrefix + "_" + type][type][slug_6]
                                            : null;
                                    }
                                }
                            }
                            else {
                                requestElement = slug;
                                slugName = requestElement.slug;
                                dataName = slugName;
                                if ('dataName' in requestElement) {
                                    dataName = requestElement.dataName;
                                }
                                if (('meta' in requestElement && requestElement.meta === true) || !metaSource) {
                                    metaSource = dataName;
                                }
                                type = 'post' in requestElement && requestElement.post === true
                                    ? 'post'
                                    : 'page';
                                asyncData[dataName] = store.state[ModulePrefix + "_" + type][type][slugName]
                                    ? store.state[ModulePrefix + "_" + type][type][slugName]
                                    : null;
                            }
                            return [2 /*return*/, asyncData];
                    }
                });
            });
        };
    }
    // META
    mixin.mixins = [
        Meta('website', metaSource) // Page
    ];
    return mixin;
});
//# sourceMappingURL=loadSections.js.map